const Redis = require('ioredis')
const {
    randomBytes
} = require('crypto')

let pools = {}

class RedisClient {
    constructor(config) {
        let {
            port,
            host,
            password,
            db,
        } = config || {}

        // let redis = null
        // let pingSi = null

        // const connect = () => {
        let redis = new Redis({
            port,
            host,
            password,
            db,
            retryStrategy: function (times) {

                console.warn('reids connection lost! retry:',times)

                var delay = Math.min(times * 50, 2000)
                return delay
            }
        })

        this.redis = redis

        redis.on('error', (e) => {
            console.error('redis connect error:', e)
        })

        // redis.ping()

        // pingSi && clearInterval(pingSi)

        // pingSi = setInterval(function () {
        //     redis.ping()
        // }, 60 * 1000)

        // }
        // connect()
    }

    static create(config, key = 'redis') {
        if (pools[key]) {
            return pools[key]
        }
        pools[key] = new RedisClient(config)

        return pools[key]
    }

    get client() {
        return this.redis
    }
}


class RedisStore {
    constructor(redis, {
        ttl,
        prefix
    } = {}) {
        this.ttl = ttl
        this.redis = redis
        this.prefix = prefix || 'SESSION'
    }

    getID(length) {
        return randomBytes(length).toString('hex')
    }

    async get(sid) {
        let id = this.prefix && sid.indexOf('_') !== 0 ? `${this.prefix}:${sid}` : sid
        let data = await this.redis.get(id)
        data && this.ttl && this.redis.expire(id, this.ttl)
        return JSON.parse(data)
    }

    async hget(key, subKey) {
        key = this.prefix && key.indexOf('_') !== 0 ? `${this.prefix}:${key}` : key
        let data = await this.redis.hget([key, subKey])
        data && this.ttl && this.redis.expire(key, this.ttl)
        return JSON.parse(data)
    }

    async set(obj, {
        sid = this.getID(24),
        ttl
    } = {}) {
        try {
            let id = this.prefix && sid.indexOf('_') !== 0 ? `${this.prefix}:${sid}` : sid
            await this.redis.set(id, JSON.stringify(obj))
            ttl = ttl || this.ttl
            ttl && this.redis.expire(id, ttl)
        } catch (e) {
            console.error(e)
        }
        return sid
    }

    async refresh(sid) {
        let id = `${this.prefix}:${sid}`
        this.ttl && this.redis.expire(id, this.ttl)
    }

    async destroy(sid) {
        return await this.redis.del(`${this.prefix}:${sid}`)
    }
}

module.exports = {
    RedisClient,
    RedisStore
}