
const debug = require('debug')('kk:session')

const {RedisClient,RedisStore} = require('./redis')

module.exports = (session = {}) => {

    let redis = RedisClient.create(session,session.key).client

    return async (ctx, next) => {

        debug('enter')

        if (!redis) {
            return await next()
        }

        const key = 'kk:session'

        const store = new RedisStore(redis,session)

        let opts = {}

        let id = ctx.cookies.get(key, opts)

        if (!id) {
            ctx.session = {}
        } else {
            ctx.session = await store.get(id, ctx)
            // check session must be a no-null object
            if (typeof ctx.session !== 'object' || ctx.session == null) {
                ctx.session = {}
            }
        }

        Object.defineProperty(ctx.session, 'refresh', {
            configurable: true,
            enumerable: false,
            value: function(){
                debug('refresh session')
                store.refresh(id)
            },
            writable: true
        })

        Object.defineProperty(ctx.session, 'save', {
            configurable: true,
            enumerable: false,
            value: function(){
                debug('saving session')
                this.pending = 'save'
            },
            writable: true
        })

        Object.defineProperty(ctx.session, 'destroy', {
            configurable: true,
            enumerable: false,
            value: function(){
                debug('destroy session')
                this.pending = 'destroy'
            },
            writable: true
        })

        Object.defineProperty(ctx.session, 'pending', {
            configurable: true,
            enumerable: false,
            value: null,
            writable: true
        })

        await next()

        if (ctx.session instanceof Object && !Object.keys(ctx.session).length) {
            ctx.session = null
        }

        if(!ctx.session){
            if(id){
                await store.destroy(id, ctx)
            }
            return
        }

        switch(ctx.session.pending){
            case 'save':{
                const sid = await store.set(ctx.session, Object.assign({}, opts, {
                    sid: id
                }), ctx)
                ctx.cookies.set(key, sid, opts)
                break
            }
            case 'destroy':{
                if(id){
                    await store.destroy(id, ctx)
                }
                break
            }
            default:{
                break
            }
        }

        // let needSave = ctx.session && ctx.session.needSave

        // // if not changed
        // if (!needSave) return

        // // if is an empty object
        // if (ctx.session instanceof Object && !Object.keys(ctx.session).length) {
        //     ctx.session = null
        // }

        // // need clear old session
        // if (id && !ctx.session) {
        //     await store.destroy(id, ctx)
        //     return
        // }

        // set/update session
        // const sid = await store.set(ctx.session, Object.assign({}, opts, {
        //     sid: id
        // }), ctx)
        // ctx.cookies.set(key, sid, opts)
    }

}