class Passport{

    constructor(){
        this._locals = {}
    }

    use(strategy){
        let name = strategy.name
        if(!name){
            return console.error('name not exist')
        }

        if(this._locals[strategy.name]){
            return console.error(strategy.name,'already exist')
        }

        this._locals[strategy.name] = strategy
        strategy.passport = this
    }

    authenticate(name,options = {}){
        let strategy = this._locals[name]
        return async (ctx,next)=>{
            try {
                let res = await strategy.verify(ctx,options)
                if(res === false){
                    return
                }
            } catch (error) {
                console.warn('passport:',name,error)
                if(options.onError){
                    await options.onError(ctx,error)
                }else{
                    ctx.$error = error
                    return await next()
                }
            }

            await next()
        }
    }

    initialize(){
        return async (ctx,next)=>{
            try {
                ctx.isAuthenticated = ()=>{
                    return ctx.session && ctx.session.profile && Object.keys(ctx.session.profile).length
                }
            } catch (error) {
                console.error(error)
            }
            await next()
            
        }
    }

    serializeUser(serializeFunc){
        this.serializeFunc = serializeFunc
    }
}

module.exports = new Passport

module.exports.Strategy = require('./strategy')

module.exports.RegisterStrategy = require('./register')

module.exports.LoginStrategy = require('./login')

module.exports.AutoLoginStrategy = require('./autoLogin')

module.exports.WeixinStrategy = require('./weixinLogin')

module.exports.WeixinOAuth = require('./weixinOAuth')