const Strategy = require('./strategy')
const querystring= require('querystring')
const uid = require('uid2')
const axios = require('axios')
const configDefault = {
    clientID: ''
    , clientSecret: ''
    , tokenURL: 'https://api.weixin.qq.com/sns/oauth2/access_token'
    , authorizationURL: 'https://open.weixin.qq.com/connect/oauth2/authorize' //[公众平台-网页授权获取用户基本信息]的授权URL 不同于[开放平台-网站应用微信登录]的授权URL
    , scope: 'snsapi_userinfo' //[公众平台-网页授权获取用户基本信息]的应用授权作用域 不同于[开放平台-网站应用微信登录]的授权URL
    , stateKey: 'kk_weixin_state'
    , userinfoURL: 'https://api.weixin.qq.com/sns/userinfo'
}

module.exports = class WeixinOAuth extends Strategy {
    constructor(config, creator){
        super('weixinOAuth')
        this.config = Object.assign(configDefault, config)
        this.creatFunc = creator
    }

    async verify(ctx,options = {}){
        if(ctx.session.wechatUserInfo){
            return
        }
        const {originalUrl} = ctx
        const {origin} = this.config
        const callbackURL = `${origin}${originalUrl}`

        if (!this.config || !this.config.clientID || !this.config.clientSecret || !this.config.origin) {
            return Promise.reject({
                msg:'config incomplete!(clientID/clientSecret/callbackURL)'
            })
        }

        if(!this.creatFunc){
            return Promise.reject({
                msg:'function undefiend!'
            })
        }

        if (ctx.query && ctx.query.error) {
            return Promise.reject({
                msg:'weixinOAuth error!'
            })
        }

        if (ctx.query && ctx.query.code) {
            const key = this.config.stateKey
            if (!ctx.query.state || !ctx.session[key] || ctx.query.state !== ctx.session[key].state) {
                return Promise.reject({
                    msg:'Invalid authorization request state.'
                })
            }

            const params = {
                appid: this.config.clientID,
                secret: this.config.clientSecret,
                code: ctx.query.code,
                grant_type: 'authorization_code'
            }
            const accessData = ctx.session.wechatAccessToken || await this.getOAuthAccessToken(params)

            if (!accessData || accessData.errcode) {
                return Promise.reject({
                    msg:'get accessToken error',
                    des:'get accessToken error:' + accessData.errmsg
                })
            }
            const {openid, access_token} = accessData
            if(ctx.session){
                ctx.session.wechatAccessToken = accessData
                ctx.session.save()
            }

            let userInfo = await this.getUserinfo({access_token,openid})
            if(userInfo){
                ctx.session.wechatUserInfo = userInfo
                ctx.session.save()
            }
        } else {
            const params = {
                appid: this.config.clientID,
                redirect_uri: callbackURL,
                response_type: 'code',
                scope: this.config.scope,
                state: uid(24)
            }
            const key = this.config.stateKey
            if (!ctx.session[key]) {
                ctx.session[key] = {}
            }
            ctx.session[key].state = params.state
            ctx.session.save()
            const location = this.getAuthorizeUrl(params)
            ctx.redirect(location)
            console.log('weixinOAuth callbackURL :',callbackURL)
        }

        return null
    }

    getAuthorizeUrl (params) {
        const {authorizationURL} =  this.config
        let queries = []
        const order = ['appid', 'redirect_uri', 'response_type', 'scope', 'state']
        order.forEach(function(key) {
          if(params[key]) {
            let query = {}
            query[key] = params[key]
            queries.push(querystring.stringify(query))
          }
        })

      return authorizationURL + '?' + queries.join('&')
    }

    async getOAuthAccessToken (params) {
        let config = {}
        config.method = 'POST'

        const {tokenURL} =  this.config
        config.url = tokenURL

        let headers = {}
        headers['Content-Type'] = 'application/x-www-form-urlencoded'
        config.headers = headers

        let queries = []
        const order = ['appid', 'secret', 'code', 'grant_type']
        order.forEach(function(key) {
          if(params[key]) {
            let query = {}
            query[key] = params[key]
            queries.push(querystring.stringify(query))
          }
        })
        config.data = queries.join('&')

        const res = await axios(config)

        return res && res.data
    }

    async getUserinfo (params) {
        const {userinfoURL} =  this.config
        let queries = []
        const order = ['access_token', 'openid', 'lang', 'scope', 'state']
        order.forEach(function(key) {
          if(params[key]) {
            let query = {}
            query[key] = params[key]
            queries.push(querystring.stringify(query))
          }
        })
        const url = userinfoURL + '?' + queries.join('&')
        const res = await axios(url)
        return res && res.data
    }
}