const devMiddleware = (webpackDev, compiler, opts) => {
    const middleware = webpackDev(compiler, opts)
    return async (ctx, next) => {
        await middleware(ctx.req, {
            end: (content) => {
                ctx.body = content
            },
            setHeader: ctx.set.bind(ctx),
            getHeader: ctx.get.bind(ctx),
            locals: ctx.state
        }, next)
    }
}


module.exports = (app,cfg,webpack,middleware) => {

    let compiler = webpack(cfg)

    if(middleware){
        return app.use(devMiddleware(middleware,compiler,{}))
    }
    let lastPercent = 0

    if (webpack.ProgressPlugin) {
        compiler.apply(new webpack.ProgressPlugin((percentage, msg) => {
            if ((percentage - lastPercent) < 0.1) {
                return
            }
            lastPercent = percentage
            console.log('webpack:', Math.floor(percentage * 100) + '%', msg)
        }))
    }

    compiler.watch({}, (err, stats) => {
        if (err) {
            console.error(err.stack || err)
            if (err.details) {
                console.error(err.details)
            }
            return
        }

        let flag = true

        const info = stats.toJson()

        if (stats.hasErrors()) {
            console.log(stats.toString({
                chunks: false, // Makes the build much quieter
                colors: true // Shows colors in the console
            }))
            flag = false
        }

        if (stats.hasWarnings()) {
            console.warn(info.warnings)
        }
        if (flag) {
            console.log('compile success')
            lastPercent = 0
        }

    })
}